package com.ematou.wxservice.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ematou.wxservice.common.constant.WeChatConstant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;

/**
 * 微信API请求工具类
 * @author lhm
 * @version 1.0
 * 2021-05-13 11:10
 */
@Component
public class WeChatApiRestTemplate {

    private static final Logger logger = LoggerFactory.getLogger(WeChatApiRestTemplate.class);

    @Autowired
    RestTemplate restTemplate;

    /**
     * 请求微信API，返回体带有errcode和errmsg字段的调这个方法
     * @param url       url地址
     * @param params    参数
     * @return          响应信息
     */
    public String getForRest(String url, Object...params) {
        String res;
        Integer errcode;
        String errmsg;
        try {
            res = restTemplate.getForObject(url, String.class, params);

            JSONObject response = JSON.parseObject(res);
            errcode = response.getObject(WeChatConstant.RESPONSE_ERROR_CODE, Integer.class);
            errmsg = response.getObject(WeChatConstant.RESPONSE_ERROR_MSG, String.class);
            if (errcode != 0 || !errmsg.equalsIgnoreCase(WeChatConstant.RESPONSE_OK)) {
                logger.error("请求微信API失败，请求方法：GET\nurl：\n" + url + "\n请求参数：" + Arrays.toString(params) + "\nerror code：" + errcode + "\nerror message：" + errmsg);
            }
        } catch (Exception e) {
            logger.error("请求微信API失败，请求方法：GET\nurl：\n" + url + "\n请求参数：" + Arrays.toString(params) + "\n异常信息：" + e.getMessage());
            throw new RuntimeException("请求微信API失败，请求方法：GET\nurl：\n" + url + "\n请求参数：" + Arrays.toString(params) + "\n异常信息：" + e.getMessage());
        }
        return res;
    }

    /**
     * 返回体不带有errcode和errmsg字段
     * @param url       url地址
     * @param params    参数
     * @return          响应信息
     */
    public String getForOther(String url, Object...params) {
        String res;
        try {
            res = restTemplate.getForObject(url, String.class, params);
        } catch (Exception e) {
            logger.error("请求微信API失败，请求方法：GET\nurl：\n" + url + "\n请求参数：" + Arrays.toString(params) + "\n异常信息：" + e.getMessage());
            throw new RuntimeException("请求微信API失败，请求方法：GET\nurl：\n" + url + "\n请求参数：" + Arrays.toString(params) + "\n异常信息：" + e.getMessage());
        }
        return res;
    }

    /**
     * 请求微信API，返回体带有errcode和errmsg字段的调这个方法
     * @param url       url地址
     * @param body      请求体
     * @return          响应信息
     */
    public String postForRest(String url, Object body) {
        String res;
        Integer errcode;
        String errmsg;
        try {
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
            HttpEntity<Object> request = new HttpEntity<>(body, headers);
            res = restTemplate.postForObject(url, request, String.class);

            JSONObject response = JSON.parseObject(res);
            errcode = response.getObject(WeChatConstant.RESPONSE_ERROR_CODE, Integer.class);
            errmsg = response.getObject(WeChatConstant.RESPONSE_ERROR_MSG, String.class);
            if (errcode != 0 || !errmsg.equalsIgnoreCase(WeChatConstant.RESPONSE_OK)) {
                logger.error("请求微信API失败，请求方法：POST\nurl：\n" + url + "\n请求体：" + body + "\nerror code：" + errcode + "\nerror message：" + errmsg);
            }
        } catch (Exception e) {
            logger.error("请求微信API失败，请求方法：POST\nurl：\n" + url + "\n请求体：" + body + "\n异常信息：" + e.getMessage());
            throw new RuntimeException("请求微信API失败，请求方法：POST\nurl：\n" + url + "\n请求体：" + body + "\n异常信息：" + e.getMessage());
        }
        return res;
    }


    /**
     * 返回体不带有errcode和errmsg字段
     * @param url       url地址
     * @param body      请求体
     * @return          响应信息
     */
    public String postForOther(String url, Object body) {
        String res;
        try {
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
            HttpEntity<Object> request = new HttpEntity<>(body, headers);
            res = restTemplate.postForObject(url, request, String.class);
        } catch (Exception e) {
            logger.error("请求微信API失败，请求方法：POST\nurl：\n" + url + "\n请求体：" + body + "\n异常信息：" + e.getMessage());
            throw new RuntimeException("请求微信API失败，请求方法：POST\nurl：\n" + url + "\n请求体：" + body + "\n异常信息：" + e.getMessage());
        }
        return res;
    }

}
