package com.ematou.wxservice.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ematou.wxservice.api.WeChatApi;
import com.ematou.wxservice.api.WeChatApiRestTemplate;
import com.ematou.wxservice.common.constant.WeChatConstant;
import com.ematou.wxservice.config.WeChatGeneralConfig;
import com.ematou.wxservice.entity.vo.AccessToken;
import com.ematou.wxservice.entity.vo.BindingInfo;
import com.ematou.wxservice.entity.vo.TemplateMessage;
import com.ematou.wxservice.entity.pojo.UserInfo;
import com.ematou.wxservice.mapper.UserInfoMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * @author lhm
 * @version 1.0
 * 2021-05-10 18:04
 */
@Service
public class WeChatService {

    private final static Logger logger = LoggerFactory.getLogger(WeChatService.class);

    @Autowired
    WeChatGeneralConfig weChatGeneralConfig;

    @Autowired
    WeChatApiRestTemplate weChatApiRestTemplate;

    @Autowired
    UserInfoMapper userInfoMapper;

    /**
     * 获取普通AccessToken
     * @return AccessToken
     * @throws RuntimeException 异常
     */
    public AccessToken getAccessToken() throws RuntimeException {
        String response = weChatApiRestTemplate.getForOther(WeChatApi.GET_TOKEN.getUrl());
        AccessToken accessToken = JSON.parseObject(response).getObject("data", AccessToken.class);

        if (null == accessToken) {
            response = weChatApiRestTemplate.getForOther(WeChatApi.GENERAL_TOKEN.getUrl());
            accessToken = JSON.parseObject(response).getObject("data", AccessToken.class);
            if (null == accessToken) {
                logger.error("微信基础服务可能已经宕机！response：" + response);
                throw new RuntimeException("获取AccessToken失败，请稍后再试！");
            }
        }

        return accessToken;
    }

    /**
     * 获取用户信息
     * @param bindingInfo bindingInfo
     * @return 用户信息
     * @throws RuntimeException 异常
     */
    public UserInfo getUserInfo(BindingInfo bindingInfo) throws RuntimeException {
        if (!StringUtils.hasLength(bindingInfo.getOpenId())) {
            logger.error("获取用户信息的openId为空");
            return null;
        }
        String accessToken = getAccessToken().getAccessToken();

        UserInfo userInfo;
        String response = null;
        try {
            response = weChatApiRestTemplate.getForOther(String.format(WeChatApi.GET_USER_INFO.getUrl(), accessToken, bindingInfo.getOpenId()));
            userInfo = JSON.parseObject(response, UserInfo.class);
            userInfo.setTellPhoneNumber(bindingInfo.getTellPhoneNumber());
            userInfo.setValidateCode(bindingInfo.getValidateCode());
            userInfoMapper.updateOrInsertUserInfo(userInfo);
        } catch (RuntimeException e) {
            logger.error("请求微信平台获取用户信息出错！response：" + response + "，error message：" + e.getMessage());
            throw new RuntimeException("请求微信平台获取用户信息出错！");
        }
        return userInfo;
    }

    /**
     * 根据模板编号获取模板消息ID
     * @param templateNum 模板编号
     * @see WeChatConstant.TemplateMsgId
     * @return 模板消息id
     */
    public String getTemplateMsgId(String templateNum){

        String url = String.format(WeChatApi.GET_TEMPLATE_MSGID.getUrl(), getAccessToken().getAccessToken());

        String requestBody = "{ \"template_id_short\": \"" + templateNum + "\" }";

        String res = weChatApiRestTemplate.postForRest(url, requestBody);

        if (StringUtils.hasLength(res)) {
            JSONObject response = JSON.parseObject(res);
            String templateId = response.getObject("template_id", String.class);
            logger.info("获取模板消息ID成功，template_id：" + templateId);
            return templateId;
        }
        return "";
    }

    /**
     * 发送取件模板信息
     * @param msg 模板信息数据
     * @return 是否发送成功
     */
    public Integer sendTemplateMsg(TemplateMessage msg){

        String accessToken = getAccessToken().getAccessToken();
        if (!StringUtils.hasLength(accessToken)) {
            logger.error("发送模板消息时获取AccessToken值为空");
            return 0;
        }
        // TODO 这里先写死，后期可以通过请求参数传过来，由客户端决定发送什么消息
        String templateMsgId = WeChatConstant.TemplateMsgId.TAKE_PARCEL_CODE_TEMPLATE_MSG_ID;

        String url = String.format(WeChatApi.SEND_TEMPLATE_MSG.getUrl(), accessToken);

        // 组装消息数据
        HashMap<String, Object> body = new HashMap<>();
        body.put("touser", msg.getOpenId());
        body.put("template_id", templateMsgId);
        if (StringUtils.hasLength(msg.getUrl())) {
            body.put("url", msg.getUrl());
        }
        if (null != msg.getMiniProgram()) {
            body.put("miniprogram", msg.getMiniProgram());
        }
        HashMap<String, Object> data = new HashMap<>();
        // 组装data数据
        for (int i = 0; i < msg.getData().size(); i++) {
            TemplateMessage.TemplateData templateData = msg.getData().get(i);
            if (i == 0) {
                data.put("first", templateData);
                continue;
            }
            if (i == msg.getData().size()-1) {
                data.put("remark", templateData);
                break;
            }
            data.put("keyword" + i, templateData);
        }
        body.put("data", data);

        String res;
        try {
            res = weChatApiRestTemplate.postForRest(url, body);

            if (!StringUtils.hasLength(res)) {
                return 0;
            }
        } catch (Exception e) {
            logger.error("推送模板消息失败，exception message：" + e.getMessage());
            return 0;
        }

        return 1;
    }

    /**
     * 微信OAuth2.0授权，静默授权只获取openid
     * @param code 请求参数, 临时凭证
     * @return openid
     */
    public Map<String, Object> oauth2(String code){

        HashMap<String, Object> map = new HashMap<>();
        String url = String.format(WeChatApi.WECHAT_OAUTH_TOKEN.getUrl(), weChatGeneralConfig.getAppId(), weChatGeneralConfig.getAppSecret(), code);

        String res = weChatApiRestTemplate.getForOther(url);

        if (StringUtils.hasLength(res)) {

            JSONObject jsonObject = JSON.parseObject(res);

            Object openid = jsonObject.get("openid");

            if (null != openid && StringUtils.hasLength(openid.toString())) {
                map.put("openid", openid);
            } else {
                logger.error("授权出现错误! 错误信息: " + res);
            }
        }

        return map;
    }
}
